@@ -35,6 +35,7 @@ from typing import Dict, List
 
 from adi.context_manager import context_manager
 from adi.rx_tx import rx_tx
+from adi.sync_start import sync_start
 
 
 def _map_to_dict(paths, ch):
@@ -78,8 +79,8 @@ def _sortconv(chans_names, noq=False, dds=False):
     return chans_names_out
 
 
-class ad9081(rx_tx, context_manager):
-    """ AD9081 Mixed-Signal Front End (MxFE) """
+class ad9081(rx_tx, context_manager, sync_start):
+    """AD9081 Mixed-Signal Front End (MxFE)"""
 
     _complex_data = True
     _rx_channel_names: List[str] = []
@@ -148,6 +149,7 @@ class ad9081(rx_tx, context_manager):
                     self._tx_fine_duc_channel_names += channels
 
         rx_tx.__init__(self)
+        sync_start.__init__(self)
         self.rx_buffer_size = 2 ** 16
 
     def _get_iio_attr_str_single(self, channel_name, attr, output):
@@ -174,17 +176,25 @@ class ad9081(rx_tx, context_manager):
         # This is overridden by subclasses
         return self._set_iio_dev_attr(attr, value)
 
+    def _get_iio_dev_attr_str_single(self, attr):
+        # This is overridden by subclasses
+        return self._get_iio_dev_attr_str(attr)
+
+    def _set_iio_dev_attr_str_single(self, attr, value):
+        # This is overridden by subclasses
+        return self._set_iio_dev_attr_str(attr, value)
+
     @property
     def path_map(self):
-        """ path_map: Map of channelizers both coarse and fine to
-            individual driver channel names
+        """path_map: Map of channelizers both coarse and fine to
+        individual driver channel names
         """
         return self._path_map
 
     def write_pfilt_config(self, value):
-        """ Load a new PFILT configuration file
-            Input is path to PFILT configuration file. Please see
-            driver documentation about PFILT generation and limitations
+        """Load a new PFILT configuration file
+        Input is path to PFILT configuration file. Please see
+        driver documentation about PFILT generation and limitations
         """
         with open(value, "r") as file:
             data = file.read()
@@ -195,8 +205,7 @@ class ad9081(rx_tx, context_manager):
 
     @property
     def rx_channel_nco_frequencies(self):
-        """rx_channel_nco_frequencies: Receive path fine DDC NCO frequencies
-        """
+        """rx_channel_nco_frequencies: Receive path fine DDC NCO frequencies"""
         return self._get_iio_attr_vec(
             self._rx_fine_ddc_channel_names, "channel_nco_frequency", False
         )
@@ -209,8 +218,7 @@ class ad9081(rx_tx, context_manager):
 
     @property
     def rx_channel_nco_phases(self):
-        """rx_channel_nco_phases: Receive path fine DDC NCO phases
-        """
+        """rx_channel_nco_phases: Receive path fine DDC NCO phases"""
         return self._get_iio_attr_vec(
             self._rx_fine_ddc_channel_names, "channel_nco_phase", False
         )
@@ -223,8 +231,7 @@ class ad9081(rx_tx, context_manager):
 
     @property
     def rx_main_nco_frequencies(self):
-        """rx_main_nco_frequencies: Receive path coarse DDC NCO frequencies
-        """
+        """rx_main_nco_frequencies: Receive path coarse DDC NCO frequencies"""
         return self._get_iio_attr_vec(
             self._rx_coarse_ddc_channel_names, "main_nco_frequency", False
         )
@@ -237,8 +244,7 @@ class ad9081(rx_tx, context_manager):
 
     @property
     def rx_main_nco_phases(self):
-        """rx_main_nco_phases: Receive path coarse DDC NCO phases
-        """
+        """rx_main_nco_phases: Receive path coarse DDC NCO phases"""
         return self._get_iio_attr_vec(
             self._rx_coarse_ddc_channel_names, "main_nco_phase", False
         )
@@ -251,7 +257,7 @@ class ad9081(rx_tx, context_manager):
 
     @property
     def rx_test_mode(self):
-        """rx_test_mode: NCO Test Mode """
+        """rx_test_mode: NCO Test Mode"""
         return self._get_iio_attr_str_single("voltage0_i", "test_mode", False)
 
     @rx_test_mode.setter
@@ -262,19 +268,117 @@ class ad9081(rx_tx, context_manager):
 
     @property
     def rx_nyquist_zone(self):
-        """rx_nyquist_zone: ADC nyquist zone. Options are: odd, even """
-        return self._get_iio_attr_str_single("voltage0_i", "nyquist_zone", False)
+        """rx_nyquist_zone: ADC nyquist zone. Options are: odd, even"""
+        return self._get_iio_attr_str_vec(
+            self._rx_coarse_ddc_channel_names, "nyquist_zone", False
+        )
 
     @rx_nyquist_zone.setter
     def rx_nyquist_zone(self, value):
-        self._set_iio_attr_single(
-            "voltage0_i", "nyquist_zone", False, value,
+        self._set_iio_attr_str_vec(
+            self._rx_coarse_ddc_channel_names, "nyquist_zone", False, value,
+        )
+
+    @property
+    def rx_main_6dB_digital_gains(self):
+        """rx_main_6dB_digital_gains: Enable 6dB of gain per CDDC"""
+        return self._get_iio_attr_vec(
+            self._rx_coarse_ddc_channel_names, "main_6db_digital_gain_en", False
+        )
+
+    @rx_main_6dB_digital_gains.setter
+    def rx_main_6dB_digital_gains(self, value):
+        self._set_iio_attr_int_vec(
+            self._rx_coarse_ddc_channel_names, "main_6db_digital_gain_en", False, value,
+        )
+
+    @property
+    def rx_channel_6dB_digital_gains(self):
+        """rx_channel_6dB_digital_gains: Enable 6dB of gain per FDDC"""
+        return self._get_iio_attr_vec(
+            self._rx_fine_ddc_channel_names, "channel_6db_digital_gain_en", False
+        )
+
+    @rx_channel_6dB_digital_gains.setter
+    def rx_channel_6dB_digital_gains(self, value):
+        self._set_iio_attr_int_vec(
+            self._rx_fine_ddc_channel_names,
+            "channel_6db_digital_gain_en",
+            False,
+            value,
+        )
+
+    @property
+    def rx_main_nco_ffh_index(self):
+        """rx_main_nco_ffh_index: Receive path coarse DDC NCO index in range [0,15]"""
+        return self._get_iio_attr_vec(
+            self._rx_coarse_ddc_channel_names, "main_nco_ffh_index", False
+        )
+
+    @rx_main_nco_ffh_index.setter
+    def rx_main_nco_ffh_index(self, value):
+        self._set_iio_attr_int_vec(
+            self._rx_coarse_ddc_channel_names, "main_nco_ffh_index", False, value,
+        )
+
+    @property
+    def rx_main_nco_ffh_select(self):
+        """rx_main_nco_ffh_select: Receive path coarse DDC NCO select in range [0,15]"""
+        return self._get_iio_attr_vec(
+            self._rx_coarse_ddc_channel_names, "main_nco_ffh_select", False
+        )
+
+    @rx_main_nco_ffh_select.setter
+    def rx_main_nco_ffh_select(self, value):
+        self._set_iio_attr_int_vec(
+            self._rx_coarse_ddc_channel_names, "main_nco_ffh_select", False, value,
+        )
+
+    @property
+    def rx_main_ffh_mode(self):
+        """rx_main_ffh_mode: ADC FFH mode. Options are:
+        instantaneous_update, synchronous_update_by_transfer_bit,
+        synchronous_update_by_gpio
+        """
+        return self._get_iio_attr_str_vec(
+            self._rx_coarse_ddc_channel_names, "main_ffh_mode", False
+        )
+
+    @rx_main_ffh_mode.setter
+    def rx_main_ffh_mode(self, value):
+        self._set_iio_attr_str_vec(
+            self._rx_coarse_ddc_channel_names, "main_ffh_mode", False, value,
+        )
+
+    @property
+    def rx_main_ffh_trig_hop_en(self):
+        """rx_main_ffh_trig_hop_en: Enable triggered hopping for CDDC NCO"""
+        return self._get_iio_attr_vec(
+            self._rx_coarse_ddc_channel_names, "main_ffh_trig_hop_en", False
+        )
+
+    @rx_main_ffh_trig_hop_en.setter
+    def rx_main_ffh_trig_hop_en(self, value):
+        self._set_iio_attr_int_vec(
+            self._rx_coarse_ddc_channel_names, "main_ffh_trig_hop_en", False, value,
+        )
+
+    @property
+    def rx_main_ffh_gpio_mode_enable(self):
+        """rx_main_ffh_gpio_mode_enable: Enablles GPIO controlled frequency hopping"""
+        return self._get_iio_attr_vec(
+            self._rx_coarse_ddc_channel_names, "main_ffh_gpio_mode_en", False
+        )
+
+    @rx_main_ffh_gpio_mode_enable.setter
+    def rx_main_ffh_gpio_mode_enable(self, value):
+        self._set_iio_attr_int_vec(
+            self._rx_coarse_ddc_channel_names, "main_ffh_gpio_mode_en", False, value,
         )
 
     @property
     def tx_channel_nco_frequencies(self):
-        """tx_channel_nco_frequencies: Transmit path fine DUC NCO frequencies
-        """
+        """tx_channel_nco_frequencies: Transmit path fine DUC NCO frequencies"""
         return self._get_iio_attr_vec(
             self._tx_fine_duc_channel_names, "channel_nco_frequency", True
         )
@@ -287,8 +391,7 @@ class ad9081(rx_tx, context_manager):
 
     @property
     def tx_channel_nco_phases(self):
-        """tx_channel_nco_phases: Transmit path fine DUC NCO phases
-        """
+        """tx_channel_nco_phases: Transmit path fine DUC NCO phases"""
         return self._get_iio_attr_vec(
             self._tx_fine_duc_channel_names, "channel_nco_phase", True
         )
@@ -301,8 +404,7 @@ class ad9081(rx_tx, context_manager):
 
     @property
     def tx_channel_nco_test_tone_en(self):
-        """tx_channel_nco_test_tone_en: Transmit path fine DUC NCO test tone enable
-        """
+        """tx_channel_nco_test_tone_en: Transmit path fine DUC NCO test tone enable"""
         return self._get_iio_attr_vec(
             self._tx_coarse_duc_channel_names, "channel_nco_test_tone_en", True
         )
@@ -315,8 +417,7 @@ class ad9081(rx_tx, context_manager):
 
     @property
     def tx_channel_nco_test_tone_scales(self):
-        """tx_channel_nco_test_tone_scales: Transmit path fine DUC NCO test tone scale
-        """
+        """tx_channel_nco_test_tone_scales: Transmit path fine DUC NCO test tone scale"""
         return self._get_iio_attr_vec(
             self._tx_coarse_duc_channel_names, "channel_nco_test_tone_scale", True
         )
@@ -332,8 +433,7 @@ class ad9081(rx_tx, context_manager):
 
     @property
     def tx_channel_nco_gain_scales(self):
-        """tx_channel_nco_gain_scales Transmit path fine DUC NCO gain scale
-        """
+        """tx_channel_nco_gain_scales Transmit path fine DUC NCO gain scale"""
         return self._get_iio_attr_vec(
             self._tx_coarse_duc_channel_names, "channel_nco_gain_scale", True
         )
@@ -346,8 +446,7 @@ class ad9081(rx_tx, context_manager):
 
     @property
     def tx_main_nco_frequencies(self):
-        """tx_main_nco_frequencies: Transmit path coarse DUC NCO frequencies
-        """
+        """tx_main_nco_frequencies: Transmit path coarse DUC NCO frequencies"""
         return self._get_iio_attr_vec(
             self._tx_coarse_duc_channel_names, "main_nco_frequency", True
         )
@@ -360,8 +459,7 @@ class ad9081(rx_tx, context_manager):
 
     @property
     def tx_main_nco_phases(self):
-        """tx_main_nco_phases: Transmit path coarse DUC NCO phases
-        """
+        """tx_main_nco_phases: Transmit path coarse DUC NCO phases"""
         return self._get_iio_attr_vec(
             self._tx_coarse_duc_channel_names, "main_nco_phase", True
         )
@@ -374,8 +472,7 @@ class ad9081(rx_tx, context_manager):
 
     @property
     def tx_main_nco_test_tone_en(self):
-        """tx_main_nco_test_tone_en: Transmit path coarse DUC NCO test tone enable
-        """
+        """tx_main_nco_test_tone_en: Transmit path coarse DUC NCO test tone enable"""
         return self._get_iio_attr_vec(
             self._tx_coarse_duc_channel_names, "main_nco_test_tone_en", True
         )
@@ -388,8 +485,7 @@ class ad9081(rx_tx, context_manager):
 
     @property
     def tx_main_nco_test_tone_scales(self):
-        """tx_main_nco_test_tone_scales: Transmit path coarse DUC NCO test tone scale
-        """
+        """tx_main_nco_test_tone_scales: Transmit path coarse DUC NCO test tone scale"""
         return self._get_iio_attr_vec(
             self._tx_coarse_duc_channel_names, "main_nco_test_tone_scale", True
         )
@@ -403,48 +499,119 @@ class ad9081(rx_tx, context_manager):
     @property
     def tx_main_ffh_frequency(self):
         """tx_main_ffh_frequency: Transmitter fast frequency hop frequency. This will set
-            The NCO frequency of the NCO selected from the bank defined by tx_main_ffh_index
+        The NCO frequency of the NCO selected from the bank defined by tx_main_ffh_index
         """
-        return self._get_iio_attr_single("voltage0_i", "main_ffh_frequency", True)
+        return self._get_iio_attr_vec(
+            self._tx_coarse_duc_channel_names, "main_nco_ffh_frequency", True
+        )
 
     @tx_main_ffh_frequency.setter
     def tx_main_ffh_frequency(self, value):
-        if self.tx_main_ffh_index == 0:
-            raise Exception(
-                "To set a FFH NCO bank frequency, tx_main_ffh_index must > 0"
-            )
-        self._set_iio_attr_single(
-            "voltage0_i", "main_ffh_frequency", True, value,
+        self._set_iio_attr_int_vec(
+            self._tx_coarse_duc_channel_names, "main_nco_ffh_frequency", True, value,
         )
 
     @property
     def tx_main_ffh_index(self):
-        """tx_main_ffh_index: Transmitter fast frequency hop NCO bank index
-        """
-        return self._get_iio_attr_single("voltage0_i", "main_ffh_index", True)
+        """tx_main_ffh_index: Transmitter fast frequency hop NCO bank index  in range [0,30]"""
+        return self._get_iio_attr_vec(
+            self._tx_coarse_duc_channel_names, "main_nco_ffh_index", True
+        )
 
     @tx_main_ffh_index.setter
     def tx_main_ffh_index(self, value):
-        self._set_iio_attr_single(
-            "voltage0_i", "main_ffh_index", True, value,
+        self._set_iio_attr_int_vec(
+            self._tx_coarse_duc_channel_names, "main_nco_ffh_index", True, value,
+        )
+
+    @property
+    def tx_main_nco_ffh_select(self):
+        """tx_main_nco_ffh_select: Transmit path coarse DDC NCO select in range [0,30]"""
+        return self._get_iio_attr_vec(
+            self._tx_coarse_duc_channel_names, "main_nco_ffh_select", True
+        )
+
+    @tx_main_nco_ffh_select.setter
+    def tx_main_nco_ffh_select(self, value):
+        self._set_iio_attr_int_vec(
+            self._tx_coarse_duc_channel_names, "main_nco_ffh_select", True, value,
         )
 
     @property
     def tx_main_ffh_mode(self):
         """tx_main_ffh_mode: Set hop transition mode of NCOs Options are:
-            phase_continuous, phase_incontinuous, and phase_coherent
+        phase_continuous, phase_incontinuous, and phase_coherent
         """
-        return self._get_iio_attr_str_single("voltage0_i", "main_ffh_mode", True)
+        return self._get_iio_attr_str_vec(
+            self._tx_coarse_duc_channel_names, "main_ffh_mode", True
+        )
 
     @tx_main_ffh_mode.setter
     def tx_main_ffh_mode(self, value):
+        self._set_iio_attr_str_vec(
+            self._tx_coarse_duc_channel_names, "main_ffh_mode", True, value,
+        )
+
+    @property
+    def tx_main_ffh_gpio_mode_enable(self):
+        """tx_main_ffh_gpio_mode_enable: Enablles GPIO controlled frequency hopping"""
+        return self._get_iio_attr_single("voltage0_i", "main_ffh_gpio_mode_en", True)
+
+    @tx_main_ffh_gpio_mode_enable.setter
+    def tx_main_ffh_gpio_mode_enable(self, value):
         self._set_iio_attr_single(
-            "voltage0_i", "main_ffh_mode", True, value,
+            "voltage0_i", "main_ffh_gpio_mode_en", True, value,
+        )
+
+    @property
+    def tx_dac_en(self):
+        """tx_dac_en: Enable DACs"""
+        return self._get_iio_attr_vec(self._tx_coarse_duc_channel_names, "en", True)
+
+    @tx_dac_en.setter
+    def tx_dac_en(self, value):
+        self._set_iio_attr_int_vec(
+            self._tx_coarse_duc_channel_names, "en", True, value,
+        )
+
+    def set_tx_dac_full_scale_current(self, value):
+        """tx_dac_full_scale_current: Set full scale current of DACs. This value
+        is in microamps.
+        """
+        self._set_iio_debug_attr_str(
+            "dac-full-scale-current-ua", str(value), self._rxadc
+        )
+
+    # we cannot read current as the driver will just throw EPERM
+    tx_dac_full_scale_current = property(None, set_tx_dac_full_scale_current)
+
+    @property
+    def powerdown(self):
+        """powerdown: Powerdown and reset the chip
+
+        Support for dynamic powerdown. Writing device attribute
+        powerdown with 'Yy1Nn0', or [oO][NnFf] for "on" and "off", will either
+        stop the jesd204 fsm, reset the device and power down an optional
+        regulator (vdd), or do the opposite in reverse order.
+        """
+        return self._get_iio_dev_attr_single("powerdown")
+
+    @powerdown.setter
+    def powerdown(self, value):
+        self._set_iio_dev_attr_single(
+            "powerdown", value,
         )
 
     @property
     def loopback_mode(self):
         """loopback_mode: Enable loopback mode RX->TX
+
+        When enabled JESD RX FIFO is connected to JESD TX FIFO,
+        making the entire datasource for the TX path the RX path. No
+        data is passed into the TX path from off-chip when 1. For
+        this mode to function correctly the JESD configuration
+        between RX and TX must be identical and only use a single
+        link.
         """
         return self._get_iio_dev_attr_single("loopback_mode")
 
@@ -454,6 +621,21 @@ class ad9081(rx_tx, context_manager):
             "loopback_mode", value,
         )
 
+    @property
+    def tx_ddr_offload(self):
+        """tx_ddr_offload: Enable DDR offload
+
+        When true the DMA will pass buffers into the BRAM FIFO for data repeating.
+        This is necessary when operating at high DAC sample rates. This can reduce
+        the maximum buffer size but data passed to DACs in cyclic mode will not
+        underflow due to memory bottlenecks.
+        """
+        return self._get_iio_debug_attr("pl_ddr_fifo_enable", self._txdac)
+
+    @tx_ddr_offload.setter
+    def tx_ddr_offload(self, value):
+        self._set_iio_debug_attr_str("pl_ddr_fifo_enable", str(value * 1), self._txdac)
+
     @property
     def rx_sample_rate(self):
         """rx_sampling_frequency: Sample rate after decimation"""
@@ -473,3 +655,75 @@ class ad9081(rx_tx, context_manager):
     def dac_frequency(self):
         """dac_frequency: DAC frequency in Hz"""
         return self._get_iio_attr_single("voltage0_i", "dac_frequency", True)
+
+    @property
+    def jesd204_fsm_ctrl(self):
+        """jesd204_fsm_ctrl: jesd204-fsm control"""
+        return self._get_iio_dev_attr("jesd204_fsm_ctrl", self._rxadc)
+
+    @jesd204_fsm_ctrl.setter
+    def jesd204_fsm_ctrl(self, value):
+        self._set_iio_dev_attr("jesd204_fsm_ctrl", value, self._rxadc)
+
+    @property
+    def jesd204_fsm_resume(self):
+        """jesd204_fsm_resume: jesd204-fsm resume"""
+        return self._get_iio_dev_attr("jesd204_fsm_resume", self._rxadc)
+
+    @jesd204_fsm_resume.setter
+    def jesd204_fsm_resume(self, value):
+        self._set_iio_dev_attr_str("jesd204_fsm_resume", value, self._rxadc)
+
+    @property
+    def jesd204_fsm_state(self):
+        """jesd204_fsm_state: jesd204-fsm state"""
+        return self._get_iio_dev_attr_str("jesd204_fsm_state", self._rxadc)
+
+    @property
+    def jesd204_fsm_paused(self):
+        """jesd204_fsm_paused: jesd204-fsm paused"""
+        return self._get_iio_dev_attr("jesd204_fsm_paused", self._rxadc)
+
+    @property
+    def jesd204_fsm_error(self):
+        """jesd204_fsm_error: jesd204-fsm error"""
+        return self._get_iio_dev_attr("jesd204_fsm_error", self._rxadc)
+
+    @property
+    def jesd204_device_status(self):
+        """jesd204_device_status: Device jesd204 link status information"""
+        return self._get_iio_debug_attr_str("status", self._rxadc)
+
+    @property
+    def jesd204_device_status_check(self):
+        """jesd204_device_status_check: Device jesd204 link status check
+
+        Returns 'True' in case error conditions are detected, 'False' otherwise
+        """
+        stat = self._get_iio_debug_attr_str("status", self._rxadc)
+
+        for s in stat.splitlines(0):
+            if "JRX" in s:
+                if "204C" in s:
+                    if "Link is good" not in s:
+                        return True
+                elif "204B" in s:
+                    if "0x0 lanes in DATA" in s:
+                        return True
+            elif "JTX" in s:
+                if any(
+                    substr in s
+                    for substr in [" asserted", "unlocked", "lost", "invalid"]
+                ):
+                    return True
+        return False
+
+    @property
+    def chip_version(self):
+        """chip_version: Chip version information"""
+        return self._get_iio_debug_attr_str("chip_version", self._rxadc)
+
+    @property
+    def api_version(self):
+        """api_version: API version"""
+        return self._get_iio_debug_attr_str("api_version", self._rxadc)
