@@ -54,6 +54,8 @@ class dds(attribute):
                 split_cores_indx = split_cores_indx + 1
             if not chan:
                 return
+            if indx >= len(value):
+                return
             if attr == "raw":
                 chan.attrs[attr].value = str(int(value[indx]))
             else:
@@ -79,6 +81,7 @@ class dds(attribute):
         return values
 
     def disable_dds(self):
+        """Disable all DDS channels and set all output sources to zero."""
         self.dds_enabled = np.zeros(self._num_tx_channels * 2, dtype=bool)
 
     @property
